/*-
 * Copyright 2014 Jason Harmening
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

#ifndef __CX88_JSON_H__
#define __CX88_JSON_H__

#include <json.h>
#include "cx88_debug.h"

struct cx88_json_object
{
   cx88_json_object(const char *str)
   {
      obj = json_tokener_parse(str);
      if (obj == NULL) CX88ERR << "Malformed JSON string" << std::endl;
      else if (json_object_get_type(obj) != json_type_object) CX88ERR << "JSON string does not represent an object" << std::endl;
   }

   ~cx88_json_object()
   {
      json_object_put(obj);
   }

   template <typename T>
   inline void get_field(const char *field, T& value)
   {
      json_object *val = NULL;
      json_object_object_get_ex(obj, field, &val);
      if (val == NULL) return;
      int64_t num;
      switch (json_object_get_type(val))
      {
         case json_type_string:
         {
            const char *str = json_object_get_string(val);
            char *endptr = NULL;
            num = strtol(str, &endptr, 0);
            if (endptr != (str + json_object_get_string_len(val)))
            {
               CX88ERR << "Malformed value for field " << field << std::endl;
               return;
            }
            break;
         }
         case json_type_int:
            num = json_object_get_int64(val);
            break;
         default:
            CX88ERR << "Invalid type for field " << field << std::endl;
            return;
      }
      value = (T)num;
   }
   
   json_object *obj;
};

#define CX88_JSON_GET_FIELD(jsonobj, container, fieldName) \
   jsonobj.get_field<typeof(container.fieldName)>(#fieldName, container.fieldName)

#define CX88_JSON_GET_STREAM(container, fieldName) \
   "\"" << #fieldName << "\":\"0x" << hex << (unsigned long)container.fieldName << dec << "\""

#endif

